#include <iostream>
#include <iomanip>
#include <vector>
#include <math.h>
#include <cstdlib>

using namespace std;

double r_pd(double pin, double ein, double mt);
double r_pout(double pin, double ein, double mt, double thetad);
double r_ed(double pin, double ein, double mt);
double r_q(double pin, double pd, double thetad);
double r_thetad(double theta, double pin, double ein, double mt);

double mu(931.494);
double MEp(7.289); // mass excess of p
double MEli(14.908); // mass excess of li

static const double PI(3.14159265358);

int main(int argc, char* argv[])
{
  double mp(MEp + mu * 1);
  double mt(MEli + mu * 7);
 
  //  double k(230.0);
  //  double k(100.0);
  //  double k(250.0);
  if(argc != 2){
    cout << "input energy" << endl;
    return -1;
  }
  double k(atoi(argv[1]));
  double pin(sqrt(k*k + 2.0*k*mp));
  double ein(sqrt(pin*pin + mp*mp));
  double theta(0.0);

  //  cout << pin << ' ' << ein << endl;

  cout << endl;
  cout << "start calculation of Li(p,n)Be event cross section" << endl;
  cout << "incident kinetic energy = " << k << " MeV" << endl;
  cout << endl;

  double dtheta = 0.01;
  double theta_cm_last = 0;

  cout << "rad(lab), "  << flush;   
  cout << "rad(cm), "  << flush;   
  cout << "m.t"   << "(fm-1),"   << flush;   
  cout << "sigma"   << "(mb/sr),"  << flush;   
  cout << "sigma"   << "(mb),"     << flush;   
  cout << "sigma_sum"   << "(mb)"     << endl;

  //  for(theta = 0.00; theta < 0.101; theta += dtheta){
  //  for(theta = 0.00; theta < 0.101; theta += dtheta){
  for(theta = 0.00; theta < 0.26; theta += dtheta){
    double pd = r_pd(pin, ein, mt);
    double thetad = r_thetad(theta, pin, ein, mt);
    double pout = r_pout(pin, ein, mt, theta);
    //    double q = r_q(pin, pout, thetad);
    double q = r_q(pin, pout, theta);
    double rrt = 5.7;//200-300MeV
    //    double sigmao = 26.9;//160MeV
    double sigmao = 26.0;//200MeV
    //    double sigmao = 27.0;//494MeV
    double sigma = sigmao*exp(-q*q*rrt);

    double theta_cm = thetad;
    double dtheta_cm = theta_cm-theta_cm_last;
    theta_cm_last = theta_cm;

    double sigmastr = sigma*2*PI*sin(theta_cm-dtheta_cm/2)*dtheta_cm;
    static double sigmasum(0);
    sigmasum += sigmastr;

    cout.setf(ios::scientific);
    cout << " " << setprecision(2) << theta    << flush;
    cout << " " << setprecision(2) << thetad   << flush;
    cout << " " << setprecision(2) << q        << flush;
    cout << " " << setprecision(2) << sigma    << flush;
    cout << " " << setprecision(2) << sigmastr << flush;
    cout << " " << setprecision(2) << sigmasum << flush;
    cout << endl;
  }

  return 0;
}

double r_q(double pin, double pd, double thetad)
{
  return sqrt(pin*pin + pd*pd - 2.0*pin*pd*cos(thetad))/197.0;
}

double r_pd(double pin, double ein, double mt)
{
  double beta = pin/(ein + mt);
  double gamma = 1.0/sqrt(1.0-beta*beta);

  return -gamma * beta * ein + gamma * pin;
}

double r_pout(double pin, double ein, double mt, double theta)
{
  double beta = pin/(ein + mt);
  double gamma = 1.0/sqrt(1.0-beta*beta);

  double pd = r_pd(pin, ein, mt);
  double ed = r_ed(pin, ein, mt);

  double pout_x = gamma * beta * ed + gamma * pd * cos(theta);
  double pout_y = pd * sin(theta);
  double pout = sqrt(pout_x*pout_x + pout_y*pout_y);
  
  return pout;
}


double r_ed(double pin, double ein, double mt)
{
  double beta = pin/(ein + mt);
  double gamma = 1.0/sqrt(1.0-beta*beta);

  return gamma * ein - gamma * beta * pin;
}

double r_thetad(double theta, double pin, double ein, double mt)
{
  double beta = pin/(ein + mt);
  double gamma = 1.0/sqrt(1.0-beta*beta);

  double pd = r_pd(pin, ein, mt);
  double ed = r_ed(pin, ein, mt);
  
  double a = gamma*tan(theta);
  double b = beta * ed / pd; 

  //  cout.precision(10);
  //  cout << beta << ' ' << gamma << ' ' << pd << ' ' << ed
  //       << ' ' << a << ' ' <<  b << endl;

  double x1 = a*a+1;
  double x2 = a*a*b;
  double x3 = a*a*b*b-1;
  double costhetap = (-x2+sqrt(x2*x2 - x1*x3))/x1;
  //  double costhetan = (-x2-sqrt(x2*x2 - x1*x3))/x1;

  //  cout << costhetap << ' ' << costhetan << endl;

  return acos(costhetap);
}

